@@ -10,7 +10,7 @@
 
 /************************************************************************
     FAUST Architecture File
-    Copyright (C) 2004-2011 GRAME, Centre National de Creation Musicale
+    Copyright (C) 2004-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This Architecture section is free software; you can redistribute it
     and/or modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
-#include <math.h>
 #include <errno.h>
 #include <time.h>
 #include <unistd.h>
@@ -69,12 +68,20 @@
 
 #include "faust/gui/UI.h"
 #include "faust/gui/JSONUI.h"
+#include "faust/gui/JSONUIDecoder.h"
+#include "faust/dsp/dsp-combiner.h"
 #include "faust/dsp/dsp.h"
 #include "faust/misc.h"
 
+// Always included
+#include "faust/gui/OSCUI.h"
+
 #ifdef POLY2
-#include "faust/dsp/dsp-combiner.h"
-#include "effect.cpp"
+#include "effect.h"
+#endif
+
+#if SOUNDFILE
+#include "faust/gui/SoundUI.h"
 #endif
 
 using namespace std;
@@ -107,10 +114,11 @@ using namespace std;
 #include "jpatcher_api.h"
 #include <string.h>
 
-#define ASSIST_INLET 	1  		/* should be defined somewhere ?? */
-#define ASSIST_OUTLET 	2		/* should be defined somewhere ?? */
+#define ASSIST_INLET 	1  	/* should be defined somewhere ?? */
+#define ASSIST_OUTLET 	2	/* should be defined somewhere ?? */
 
-#define EXTERNAL_VERSION "0.62"
+#define EXTERNAL_VERSION    "0.67"
+#define STR_SIZE            512
 
 #include "faust/gui/GUI.h"
 #include "faust/gui/MidiUI.h"
@@ -146,6 +154,17 @@ struct Max_Meta2 : Meta
     }
 };
 
+struct Max_Meta3 : Meta
+{
+    string fName;
+    void declare(const char* key, const char* value)
+    {
+        if ((strcmp("filename", key) == 0)) {
+            fName = "com.grame." + string(value) + "~";
+        }
+    }
+};
+
 /*--------------------------------------------------------------------------*/
 typedef struct faust
 {
@@ -165,6 +184,12 @@ typedef struct faust
     MidiUI* m_midiUI;
     midi_handler* m_midiHandler;
 #endif
+#ifdef SOUNDFILE
+    SoundUI* m_soundInterface;
+#endif
+#ifdef OSCCTRL
+    OSCUI* m_oscInterface;
+#endif
 } t_faust;
 
 void* faust_class;
@@ -187,10 +212,10 @@ public:
     mspUIObject(const string& label, FAUSTFLOAT* zone):fLabel(label),fZone(zone) {}
     virtual ~mspUIObject() {}
     
-    virtual void setValue(FAUSTFLOAT f) {*fZone = range(0.0, 1.0, f);}
+    virtual void setValue(FAUSTFLOAT f) { *fZone = range(0.0, 1.0, f); }
     virtual FAUSTFLOAT getValue() { return *fZone; }
     virtual void toString(char* buffer) {}
-    virtual string getName() {return fLabel;}
+    virtual string getName() { return fLabel; }
   
 };
 
@@ -204,7 +229,7 @@ class mspCheckButton : public mspUIObject {
         
         void toString(char* buffer)
         {
-            snprintf(buffer, 256, "CheckButton(float): %s", fLabel.c_str());
+            snprintf(buffer, STR_SIZE, "CheckButton(float): %s", fLabel.c_str());
         }
 };
 
@@ -218,7 +243,7 @@ class mspButton : public mspUIObject {
         
         void toString(char* buffer)
         {
-            snprintf(buffer, 256, "Button(float): %s", fLabel.c_str());
+            snprintf(buffer, STR_SIZE, "Button(float): %s", fLabel.c_str());
         }
 };
 
@@ -243,10 +268,10 @@ class mspSlider : public mspUIObject {
             stringstream str;
             str << "Slider(float): " << fLabel << " [init=" << fInit << ":min=" << fMin << ":max=" << fMax << ":step=" << fStep << ":cur=" << *fZone << "]";
             string res = str.str();
-            snprintf(buffer, 256, res.c_str());
+            snprintf(buffer, STR_SIZE, "%s", res.c_str());
         }
         
-        void setValue(FAUSTFLOAT f) {*fZone = range(fMin, fMax, f);}
+        void setValue(FAUSTFLOAT f) { *fZone = range(fMin, fMax, f); }
 };
 
 /*--------------------------------------------------------------------------*/
@@ -269,7 +294,7 @@ class mspBargraph : public mspUIObject {
             stringstream str;
             str << "Bargraph(float): " << fLabel << " [min=" << fMin << ":max=" << fMax << ":cur=" << *fZone << "]";
             string res = str.str();
-            snprintf(buffer, 256, res.c_str());
+            snprintf(buffer, STR_SIZE, "%s", res.c_str());
         }
         
         virtual FAUSTFLOAT getValue() 
@@ -447,7 +472,7 @@ class mspUI : public UI
             iterator it;
             post((char*)"------- labels and ranges ----------");
             for (it = fUITable1.begin(); it != fUITable1.end(); it++) {
-                char param[1024];
+                char param[STR_SIZE];
                 it->second->toString(param);
                 post(param);
             }
@@ -486,10 +511,9 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
     bool res = false;
     string name = string((s)->s_name);
     
-    // Check if no argument is there, consider it is a toggle message for a button
+    // If no argument is there, consider it as a toggle message for a button
     if (ac == 0 && obj->m_dspUI->isValue(name)) {
         
-        string name = string((s)->s_name);
         float off = 0.0f;
         float on = 1.0f;
         obj->m_dspUI->setValue(name, off);
@@ -541,89 +565,80 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
                     return;         
             }
             
-            stringstream num_val;
-            num_val << num + i;
-            string str = num_val.str();
-            char param_name[256];
-            
-            switch (ndigit - count_digit(str)) {
-                case 0:
-                    sprintf(param_name, "%s%s", prefix.c_str(), str.c_str());
-                    break;
-                case 1:
-                    sprintf(param_name, "%s %s", prefix.c_str(), str.c_str());
-                    break;
-                case 2:
-                    sprintf(param_name, "%s  %s", prefix.c_str(), str.c_str());
-                    break;
+            stringstream num_val; num_val << num + i;
+            stringstream param_name; param_name << prefix;
+            for (int i = 0; i < ndigit - count_digit(num_val.str()); i++) {
+                param_name << ' ';
             }
-            
+            param_name << num_val.str();
+                
             // Try special naming scheme for list of parameters
-            res = obj->m_dspUI->setValue(param_name, value); 
+            res = obj->m_dspUI->setValue(param_name.str(), value);
             
             // Otherwise try standard name
             if (!res) {
                 res = obj->m_dspUI->setValue(name, value);
             }
-            
             if (!res) {
                 post("Unknown parameter : %s", (s)->s_name);
             }
         }
+        
     } else {
         // Standard parameter name
         float value = (av[0].a_type == A_LONG) ? (float)av[0].a_w.w_long : av[0].a_w.w_float;
-        res = obj->m_dspUI->setValue(name, value); 
+        res = obj->m_dspUI->setValue(name, value);
+        if (!res) {
+            post("Unknown parameter : %s", (s)->s_name);
+        }
     }
     
-    if (!res) {
-        post("Unknown parameter : %s", (s)->s_name);
-    }
 }
 
 /*--------------------------------------------------------------------------*/
-void faust_polyphony(t_faust* obj, t_symbol* s, short ac, t_atom* av)
+void faust_polyphony(t_faust* x, t_symbol* s, short ac, t_atom* av)
 {
-    if (systhread_mutex_lock(obj->m_mutex) == MAX_ERR_NONE) {
+    if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
     #ifdef MIDICTRL
-        mydsp_poly* poly = dynamic_cast<mydsp_poly*>(obj->m_dsp);
+        mydsp_poly* poly = dynamic_cast<mydsp_poly*>(x->m_dsp);
         if (poly) {
-            obj->m_midiHandler->removeMidiIn(poly);
+            x->m_midiHandler->removeMidiIn(poly);
         }
     #endif
         // Delete old
-        delete obj->m_dsp;
-        obj->m_dspUI->clear();
+        delete x->m_dsp;
+        x->m_dspUI->clear();
         mydsp_poly* dsp_poly = NULL;
         // Allocate new one
         if (av[0].a_w.w_long > 0) {
             post("polyphonic DSP voices = %d", av[0].a_w.w_long);
             dsp_poly = new mydsp_poly(new mydsp(), av[0].a_w.w_long, true, true);
         #ifdef POLY2
-            obj->m_dsp = new dsp_sequencer(dsp_poly, new effect());
+            x->m_dsp = new dsp_sequencer(dsp_poly, new effect());
         #else
-            obj->m_dsp = dsp_poly;
+            x->m_dsp = dsp_poly;
         #endif
         } else {
-            obj->m_dsp = new mydsp();
+            x->m_dsp = new mydsp();
             post("monophonic DSP");
         }
         // Initialize User Interface (here connnection with controls)
-        obj->m_dsp->buildUserInterface(obj->m_dspUI);
+        x->m_dsp->buildUserInterface(x->m_dspUI);
     #ifdef MIDICTRL
-        obj->m_midiHandler->addMidiIn(dsp_poly);
-        obj->m_dsp->buildUserInterface(obj->m_midiUI);
+        x->m_midiHandler->addMidiIn(dsp_poly);
+        x->m_dsp->buildUserInterface(x->m_midiUI);
     #endif
+   
         // Initialize at the system's sampling rate
-        obj->m_dsp->init(long(sys_getsr()));
+        x->m_dsp->init(long(sys_getsr()));
         
         // Prepare JSON
-        faust_make_json(obj);
+        faust_make_json(x);
       
         // Send JSON to JS script
-        faust_create_jsui(obj);
+        faust_create_jsui(x);
         
-        systhread_mutex_unlock(obj->m_mutex);
+        systhread_mutex_unlock(x->m_mutex);
     } else {
         post("Mutex lock cannot be taken...");
     }
@@ -631,18 +646,18 @@ void faust_polyphony(t_faust* obj, t_symbol* s, short ac, t_atom* av)
 
 /*--------------------------------------------------------------------------*/
 #ifdef MIDICTRL
-void faust_midievent(t_faust* obj, t_symbol* s, short ac, t_atom* av) 
+void faust_midievent(t_faust* x, t_symbol* s, short ac, t_atom* av) 
 {
     if (ac > 0) {
         int type = (int)av[0].a_w.w_long & 0xf0;
         int channel = (int)av[0].a_w.w_long & 0x0f;
                 
         if (ac == 1) {
-            obj->m_midiHandler->handleSync(0.0, av[0].a_w.w_long);
+            x->m_midiHandler->handleSync(0.0, av[0].a_w.w_long);
         } else if (ac == 2) {
-            obj->m_midiHandler->handleData1(0.0, type, channel, av[1].a_w.w_long);
+            x->m_midiHandler->handleData1(0.0, type, channel, av[1].a_w.w_long);
         } else if (ac == 3) {
-            obj->m_midiHandler->handleData2(0.0, type, channel, av[1].a_w.w_long, av[2].a_w.w_long);
+            x->m_midiHandler->handleData2(0.0, type, channel, av[1].a_w.w_long, av[2].a_w.w_long);
         }
     }
 }
@@ -769,11 +784,81 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
 
     ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
     
+#ifdef SOUNDFILE
+    Max_Meta3 meta3;
+    x->m_dsp->metadata(&meta3);
+    string bundle_path_str = SoundUI::getBinaryPathFrom(meta3.fName);
+    if (bundle_path_str == "") {
+        post("Bundle_path cannot be found!");
+    }
+    x->m_soundInterface = new SoundUI(bundle_path_str);
+    // SoundUI has to be dispatched on all internal voices
+    if (dsp_poly) dsp_poly->setGroup(false);
+    x->m_dsp->buildUserInterface(x->m_soundInterface);
+    if (dsp_poly) dsp_poly->setGroup(true);
+#endif
+    
+#ifdef OSCCTRL
+    x->m_oscInterface = NULL;
+#endif
+    
     // Send JSON to JS script
     faust_create_jsui(x);
     return x;
 }
 
+#ifdef OSCCTRL
+// osc 'IP inport outport xmit bundle'
+void faust_osc(t_faust* x, t_symbol* s, short ac, t_atom* av)
+{
+    if (ac == 5) {
+        if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
+            
+            delete x->m_oscInterface;
+            
+            const char* argv1[32];
+            int argc1 = 0;
+            
+            argv1[argc1++] = "Faust";
+            
+            argv1[argc1++]  = "-desthost";
+            argv1[argc1++]  = atom_getsym(&av[0])->s_name;
+            
+            char inport[32];
+            snprintf(inport, 32, "%ld", long(av[1].a_w.w_long));
+            argv1[argc1++] = "-port";
+            argv1[argc1++] = inport;
+            
+            char outport[32];
+            snprintf(outport, 32, "%ld", long(av[2].a_w.w_long));
+            argv1[argc1++] = "-outport";
+            argv1[argc1++] = outport;
+            
+            char xmit[32];
+            snprintf(xmit, 32, "%ld", long(av[3].a_w.w_long));
+            argv1[argc1++] = "-xmit";
+            argv1[argc1++] = xmit;
+            
+            char bundle[32];
+            snprintf(bundle, 32, "%ld", long(av[4].a_w.w_long));
+            argv1[argc1++] = "-bundle";
+            argv1[argc1++] = bundle;
+            
+            x->m_oscInterface = new OSCUI("Faust", argc1, (char**)argv1);
+            x->m_dsp->buildUserInterface(x->m_oscInterface);
+            x->m_oscInterface->run();
+            
+            post(x->m_oscInterface->getInfos().c_str());
+            systhread_mutex_unlock(x->m_mutex);
+        } else {
+            post("Mutex lock cannot be taken...");
+        }
+    } else {
+        post("Should be : osc 'IP inport outport xmit(0|1|2) bundle(0|1)'");
+    }
+}
+#endif
+
 /*--------------------------------------------------------------------------*/
 void faust_dblclick(t_faust* x, long inlet)
 {
@@ -821,21 +906,30 @@ void faust_free(t_faust* x)
     delete x->m_midiUI;
     delete x->m_midiHandler;
 #endif
+#ifdef SOUNDFILE
+    delete x->m_soundInterface;
+#endif
+#ifdef OSCCTRL
+    delete x->m_oscInterface;
+#endif
 }
 
 /*--------------------------------------------------------------------------*/
 t_int* faust_perform(t_int* w)
 {
-	t_faust* x = (t_faust*) (w[1]);
-	long n = w[2];
-	int offset = 3;
-	AVOIDDENORMALS;
+    AVOIDDENORMALS;
+    t_faust* x = (t_faust*) (w[1]);
+    long n = w[2];
+    int offset = 3;
     if (!x->m_mute && systhread_mutex_trylock(x->m_mutex) == MAX_ERR_NONE) {
         if (x->m_dsp) {
             x->m_dsp->compute(n, ((float**)&w[offset]), ((float**)&w[offset + x->m_dsp->getNumInputs()]));
+        #ifdef OSCCTRL
+            if (x->m_oscInterface) x->m_oscInterface->endBundle();
+        #endif
             faust_update_outputs(x);
         }
-    #ifdef MIDICTRL
+    #if defined(MIDICTRL) || defined(OSCCTRL)
         GUI::updateAllGuis();
     #endif
         systhread_mutex_unlock(x->m_mutex);
@@ -873,6 +967,9 @@ extern "C" int main(void)
     // 03/11/14 : use 'anything' to handle all parameter changes
     addmess((method)faust_anything, (char*)"anything", A_GIMME, 0);
     addmess((method)faust_polyphony, (char*)"polyphony", A_GIMME, 0);
+#ifdef OSCCTRL
+    addmess((method)faust_osc, (char*)"osc", A_GIMME, 0);
+#endif
 #ifdef MIDICTRL
     addmess((method)faust_midievent, (char*)"midievent", A_GIMME, 0);
 #endif
@@ -883,7 +980,7 @@ extern "C" int main(void)
     dsp_initclass();
     
     post((char*)"Faust DSP object v%s (sample = 32 bits code = 32 bits)", EXTERNAL_VERSION);
-    post((char*)"Copyright (c) 2012-2018 Grame");
+    post((char*)"Copyright (c) 2012-2019 Grame");
     Max_Meta1 meta1;
     tmp_dsp->metadata(&meta1);
     if (meta1.fCount > 0) {
@@ -899,6 +996,3 @@ extern "C" int main(void)
 
 /********************END ARCHITECTURE SECTION (part 2/2)****************/
 
-
-
-
